from setuptools import setup 

setup(
    name="TN2601",
    version="0.5",
    description="Librería para poder chequear ejercicios propuestos del libro numérico.",
    author="Axel Osses A. & Diego Olguín W.",
    packages=["TN"],
    zip_safe=False
)